```@meta
CurrentModule = REDCap
```
# Import

Importing into a REDCap database is straightforward. The data to be imported must be presented in either `json` (Array of Dicts), `CSV`, `DataFrame`, or `xml` format. These files can be loaded from a filepath as well. The correct format must be passed along with the object. REDCap defaults to `json` if no format is given.

<b>Note</b>
Some import functions (Metadata) are only available for projects marked in development, while others have their `override` parameters disabled (arm, event).

## Records

```@docs
import_records(config::REDCap.Config, data::Any; format::String="json", type::String="flat", overwriteBehavior::String="normal", forceAutoNumber::Bool=false, dateFormat::String="YMD", returnContent::String="count", returnFormat::String="json")
```
#### Notes:

Record:
```bash
Dict{String,Any} with 16 entries:
  "sex"                   => "0"
  "age"                   => "56"
  "address"               => "168 Anderson Blvd. Quincy MA 01227"
  "height"                => "80"
  "dob"                   => "1962-04-08"
  "record_id"             => "1"
  "bmi"                   => "125"
  "comments"              => "Randomly Generated - Demographics"
  "email"                 => "ALin@aol.com"
  "first_name"            => "Alexia"
  "demographics_complete" => "0"
  "telephone"             => "(617) 882-6049"
  "weight"                => "80"
  "last_name"             => "Lin"
  "ethnicity"             => "1"
  "race"                  => "1"
```

```julia
record_data=[Dict("record_id"=>"1","first_name"=>"John")]

import_records(config, record_data)

#or from a file
import_records(config, "/src/records.csv", format="csv")
```
Which returns the number of records imported.
```bash
Dict{String,Any} with 1 entry:
  "count" => 1
```

By default, `json` is passed as the target format, but `csv`/`df`, `xml`, and `odm` formats are supported for import as well as export.

For projects with auto-numbering enabled. The next available record ID number is generated by the `generate_next_record_id()` function:
```julia
next_id = generate_next_record_id(config)
```
Which returns the number as an integer.


## Project Info

```@docs
import_project_information(config::REDCap.Config, data; format::String="json")
```
#### Notes:

The project information and settings can be changed using this function by importing a dict with some/all of the desired settings.

```bash
Dict{String,Any} with 23 entries:
  "secondary_unique_field"              => ""
  "surveys_enabled"                     => "0"
  "purpose_other"                       => ""
  "has_repeating_instruments_or_events" => 0
  "production_time"                     => ""
  "project_grant_number"                => ""
  "ddp_enabled"                         => "0"
  "project_id"                          => "21"
  "purpose"                             => "0"
  "scheduling_enabled"                  => "0"
  "randomization_enabled"               => "0"
  "project_language"                    => "English"
  "is_longitudinal"                     => 0
  "project_title"                       => "RC Test"
  "project_pi_firstname"                => ""
  "display_today_now_button"            => "1"
  "record_autonumbering_enabled"        => "1"
  "creation_time"                       => "2018-07-16 19:17:34"
  "project_notes"                       => "testing"
  "in_production"                       => "0"
  "project_irb_number"                  => ""
  "custom_record_label"                 => ""
  "project_pi_lastname"                 => ""

```

A project can be marked as "In Production" this way, by changing the `in_production` value to 1.

```julia
#Modifying existing information
info = export_project_information(config)
info["in_production"]="1"
import_project_information(config, info)
```

Similarily, a projects Grant ID can be entered and tied to the project.

## Metadata

```@docs
import_metadata(config::REDCap.Config, data; format::String="json", returnFormat::String="json")
```
#### Notes:

A projects metadata can be modified before it leaves development status using the `import_metadata()` function

An example of the "Contact Information" field:
```bash
Dict{String,Any} with 18 entries:
  "required_field"                             => ""
  "section_header"                             => "Contact Information"
  "matrix_ranking"                             => ""
  "select_choices_or_calculations"             => ""
  "field_type"                                 => "text"
  "field_note"                                 => ""
  "form_name"                                  => "demographics"
  "matrix_group_name"                          => ""
  "field_label"                                => "First Name"
  "custom_alignment"                           => ""
  "question_number"                            => ""
  "text_validation_max"                        => ""
  "text_validation_type_or_show_slider_number" => ""
  "branching_logic"                            => ""
  "field_annotation"                           => ""
  "identifier"                                 => "y"
  "text_validation_min"                        => ""
  "field_name"                                 => "first_name"

```

```julia
#Example once this works
```

## Users

```@docs
import_users(config::REDCap.Config, data; format::String="json", returnFormat::String="json")
```
#### Notes:

Users may be imported in the same way as above. User permissions are set/modified in this way.

```bash
julia> new_user

Dict{String,Any} with 32 entries:
  "design"                     => "0"
  "api_export"                 => "0"
  "user_rights"                => "0"
  "data_access_groups"         => "0"
  "data_comparison_tool"       => "0"
  "username"                   => "john_smith@email.com"
  "data_access_group_id"       => ""
  "data_export"                => "0"
  "record_create"              => "0"
  "reports"                    => "0"
  "data_import_tool"           => "0"
  "file_repository"            => "0"
  "mobile_app_download_data"   => "0"
  "mobile_app"                 => "0"
  "email"                      => ""
  "data_quality_create"        => "0"
  "record_delete"              => "0"
  "calendar"                   => "0"
  "lock_records_all_forms"     => "0"
  "firstname"                  => ""
  "expiration"                 => ""
  "data_access_group"          => ""
  "forms"                      => Dict{String,Any}(Pair{String,Any}("demographics", "0"))
  "api_import"                 => "0"
  "stats_and_charts"           => "0"
  "record_rename"              => "0"
  "lock_records_customization" => "0"
  "logging"                    => "0"
  "lock_records"               => "0"
  "data_quality_execute"       => "0"
  "manage_survey_participants" => "0"
  "lastname"                   => ""
```

```julia
#Add above user to the list of users
push!(user_list, new_user)

#Import new list into REDCap
import_users(config, user_list)
```
User Permissions:
```
Data Export: 0=No Access, 2=De-Identified, 1=Full Data Set
Form Rights: 0=No Access, 2=Read Only, 1=View records/responses and edit records (survey responses are read-only), 3=Edit survey responses
Other attribute values: 0=No Access, 1=Access.
```
## Files

```@docs
import_file(config::REDCap.Config, record::String, field::String, event::String, file::String; repeat_instance::Int=1, returnFormat::String="json")
```
#### Notes:

A specified file upload field is required to import a file. Any attempts to upload a file in a non-file field will result in error.

```julia
import_file(config, "2", "file_upload", "", "/src/example.csv")
```

## Arms

```@docs
import_arms(config::REDCap.Config, data; override::Int=0, format::String="json", returnFormat::String="json")
```
#### Notes:

Arms may be imported into REDCap by passing the name and arm number.

```bash
Dict{String,Any} with 2 entries:
  "name"    => "Arm 1"
  "arm_num" => 1
```

```julia
import_arms(config, newarm)
```

### ATTN:
Using the override command on a project may lead to loss of arms, which will remove access to any associated records.

## Events

```@docs
import_events(config::REDCap.Config, data; override::Int=0, format::String="json", returnFormat::String="json")
```
#### Notes:

Events can be passed into REDCap by passing their name and arm number, as well as any additional configurations.
```bash
Dict{String,Any} with 7 entries:
  "unique_event_name"  => "event_1_arm_1"
  "custom_event_label" => nothing
  "offset_max"         => "0"
  "arm_num"            => "1"
  "event_name"         => "Event 1"
  "day_offset"         => "1"
  "offset_min"         => "0"
```

```julia
import_events(config, event)
```
Importing events with the same name will not overwrite by default, and will instead append a character. This can be changed by passing `override=1` to overwrite existing events.


## Instrument Event Mappings

```@docs
import_instrument_event_mappings(config::REDCap.Config, data; format::String="json", returnFormat::String="json")
```
#### Notes:

The Instrument Event mapping allows REDCap to link different forms to different events easily.

```bash
Dict{String,Any} with 3 entries:
  "arm_num"           => 1
  "form"              => "demographics"
  "unique_event_name" => "event_1_arm_1"
```
New mappings can be created by specifying the arm, form, and event name to link together.
```julia
import_instrument_event_mappings(config, newmap)
```